/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/*******************************************************************************
 *  main.c - c file for main function
 *  DRV841x_FIRMWARE
 *  05/25/2022
 *  Pablo Armet
 ******************************************************************************/
#include <msp430.h>
#include "drv841x.h"
#include "Serial_Cmd_Monitor.h"

/*
 * Globals
 */
int timerUpdate = 0;                           // Timers are unused in this firmware version
unsigned int OFCount;                          // Variable for watchdog LED blink
unsigned int i=0;                              // flag for AINx state inside TimerA1 ISR
unsigned int j=0;                              // flag for BINx state inside TimerA1 ISR
unsigned int StepperStep = 0;                  // count for stepper inside ISR
void main(void)                                // Main function
{
    /* Initialize MCU peripherals*/
    initMcu();                                 // Initialize and configure MSP430G2955
    firmwareInit();
    //drv841xevm_init();                         // EVM hardware initialization
    _enable_interrupt();
    __bis_SR_register(GIE);                    // Enable MSP430G2955 global interrupts

    // GUI Composer Monitor Initialization
    ClearBufferRelatedParam();

    while (1)                                  // Wake up and execute once in approximately 4 ms watchdog timer mode interrup using VLO
    {
//        P1OUT ^= TP14;                       // Toggle TP14 for probing with oscilloscope
        gui_deviceControlUpdateTask();         // GUI controls update with EVM hardware function
        PWM_BDC_FrequencyControl();            // Sets PWM frequency and duty cycles
        Stepper_Control();                     // Control stepper motor
        StallDetectionControl();               // Stall detection loop
        //drv_startPWM();
        __bis_SR_register(LPM0_bits + GIE);    // Enter LPM0
    } //
} // end main()

/*
 * ISR routines
 */

/*
 * Timer1_A0 ISR
 */
#pragma vector=TIMER1_A1_VECTOR
__interrupt void Timer1_A1(void)
{
    /*
     * Checking TA1IV clears the interrupt flag
     */

    switch (__even_in_range(TA1IV, 10))  // Efficient switch-implementation
    {
      case TA1IV_NONE:
          break;    // No interrupt
      case TA1IV_TACCR1:
          break;    // CCR1 capture interrupt
      case TA1IV_TACCR2:
          break;    // CCR2 capture interrupt
      case TA1IV_6:
          break;    // Reserved
      case TA1IV_8:
          break;    // Reserved
      case TA1IV_TAIFG:
          if (gBrushedObj.motor_type==0)
          {
              if (gBrushedObj.step_rate == 0)         //full-step
              {
                  if (gBrushedObj.dir == 0) //cw direction
                  {
                      switch (StepperStep)
                      {
                      case 0:
                          P3OUT |= (AIN1|BIN2);
                          P3OUT &= ~(AIN2|BIN1);
                          break;
                      case 1:
                          P3OUT |= (AIN1|BIN1);
                          P3OUT &= ~(AIN2|BIN2);
                          break;
                      case 2:
                          P3OUT |= (AIN2|BIN1);
                          P3OUT &= ~(AIN1|BIN2);
                          break;
                      case 3:
                          P3OUT |= (AIN2|BIN2);
                          P3OUT &= ~(AIN1|BIN1);
                          break;
                      default:
                          break;
                      }
                  }
                  if (gBrushedObj.dir == 1) //ccw direction
                  {
                      switch (StepperStep)
                      {
                      case 0:
                          P3OUT |= (AIN2|BIN2);
                          P3OUT &= ~(AIN1|BIN1);
                          break;
                      case 1:
                          P3OUT |= (AIN2|BIN1);
                          P3OUT &= ~(AIN1|BIN2);
                          break;
                      case 2:
                          P3OUT |= (AIN1|BIN1);
                          P3OUT &= ~(AIN2|BIN2);
                          break;
                      case 3:
                          P3OUT |= (AIN1|BIN2);
                          P3OUT &= ~(AIN2|BIN1);
                          break;
                      default:
                          break;
                      }
                  }
                  StepperStep++;
                  if (StepperStep == 4)
                      StepperStep = 0;
              }
              else if (gBrushedObj.step_rate == 1)    //half-step
              {
                  if (gBrushedObj.decay_mode == 0)    // slow decay
                  {
                      if (gBrushedObj.dir == 0) //cw direction
                      {
                          switch (StepperStep)
                          {
                          case 0:
                              P3OUT |= (AIN1|AIN2|BIN2);
                              P3OUT &= ~(BIN1);
                              break;
                          case 1:
                              P3OUT |= (AIN1|BIN2);
                              P3OUT &= ~(AIN2|BIN1);
                              break;
                          case 2:
                              P3OUT |= (AIN1|BIN1|BIN2);
                              P3OUT &= ~(AIN2);
                              break;
                          case 3:
                              P3OUT |= (AIN1|BIN1);
                              P3OUT &= ~(AIN2|BIN2);
                              break;
                          case 4:
                              P3OUT |= (AIN1|AIN2|BIN1);
                              P3OUT &= ~(BIN2);
                              break;
                          case 5:
                              P3OUT |= (AIN2|BIN1);
                              P3OUT &= ~(AIN1|BIN2);
                              break;
                          case 6:
                              P3OUT |= (AIN2|BIN1|BIN2);
                              P3OUT &= ~(AIN1);
                              break;
                          case 7:
                              P3OUT |= (AIN2|BIN2);
                              P3OUT &= ~(AIN1|BIN1);
                              break;
                          default:
                              break;
                          }
                      }
                      if (gBrushedObj.dir == 1) //ccw direction
                      {
                          switch (StepperStep)
                          {
                          case 0:
                              P3OUT |= (AIN2|BIN2);
                              P3OUT &= ~(AIN1|BIN1);
                              break;
                          case 1:
                              P3OUT |= (AIN2|BIN1|BIN2);
                              P3OUT &= ~(AIN1);
                              break;
                          case 2:
                              P3OUT |= (AIN2|BIN1);
                              P3OUT &= ~(AIN1|BIN2);
                              break;
                          case 3:
                              P3OUT |= (AIN1|AIN2|BIN1);
                              P3OUT &= ~(BIN2);
                              break;
                          case 4:
                              P3OUT |= (AIN1|BIN1);
                              P3OUT &= ~(AIN2|BIN2);
                              break;
                          case 5:
                              P3OUT |= (AIN1|BIN1|BIN2);
                              P3OUT &= ~(AIN2);
                              break;
                          case 6:
                              P3OUT |= (AIN1|BIN2);
                              P3OUT &= ~(AIN2|BIN1);
                              break;
                          case 7:
                              P3OUT |= (AIN1|AIN2|BIN2);
                              P3OUT &= ~(BIN1);
                              break;
                          default:
                              break;
                          }
                      }
                  }
                  else                               // fast decay
                  {
                      if (gBrushedObj.dir == 0) //cw direction
                      {
                          switch (StepperStep)
                          {
                          case 0:
                              P3OUT |= (BIN2);
                              P3OUT &= ~(AIN1|AIN2|BIN1);
                              break;
                          case 1:
                              P3OUT |= (AIN1|BIN2);
                              P3OUT &= ~(AIN2|BIN1);
                              break;
                          case 2:
                              P3OUT |= (AIN1);
                              P3OUT &= ~(AIN2|BIN1|BIN2);
                              break;
                          case 3:
                              P3OUT |= (AIN1|BIN1);
                              P3OUT &= ~(AIN2|BIN2);
                              break;
                          case 4:
                              P3OUT |= (BIN1);
                              P3OUT &= ~(AIN1|AIN2|BIN2);
                              break;
                          case 5:
                              P3OUT |= (AIN2|BIN1);
                              P3OUT &= ~(AIN1|BIN2);
                              break;
                          case 6:
                              P3OUT |= (AIN2);
                              P3OUT &= ~(AIN1|BIN1|BIN2);
                              break;
                          case 7:
                              P3OUT |= (AIN2|BIN2);
                              P3OUT &= ~(AIN1|BIN1);
                              break;
                          default:
                              break;
                          }
                      }
                      if (gBrushedObj.dir == 1) //ccw direction
                      {
                          switch (StepperStep)
                          {
                          case 0:
                              P3OUT |= (AIN2|BIN2);
                              P3OUT &= ~(AIN1|BIN1);
                              break;
                          case 1:
                              P3OUT |= (AIN2);
                              P3OUT &= ~(AIN1|BIN1|BIN2);
                              break;
                          case 2:
                              P3OUT |= (AIN2|BIN1);
                              P3OUT &= ~(AIN1|BIN2);
                              break;
                          case 3:
                              P3OUT |= (BIN1);
                              P3OUT &= ~(AIN1|AIN2|BIN2);
                              break;
                          case 4:
                              P3OUT |= (AIN1|BIN1);
                              P3OUT &= ~(AIN2|BIN2);
                              break;
                          case 5:
                              P3OUT |= (AIN1);
                              P3OUT &= ~(AIN2|BIN1|BIN2);
                              break;
                          case 6:
                              P3OUT |= (AIN1|BIN2);
                              P3OUT &= ~(AIN2|BIN1);
                              break;
                          case 7:
                              P3OUT |= (BIN2);
                              P3OUT &= ~(AIN1|AIN2|BIN1);
                              break;
                          default:
                              break;
                          }
                      }
                  }
                  StepperStep++;
                  if (StepperStep == 8)
                      StepperStep = 0;
              }
          }
          TA1CCTL0 &= ~CCIFG;
    }
} // end Timer1_A1()

/*
 * Timer0_A1
 */
#pragma vector=TIMER0_A1_VECTOR
__interrupt void Timer0_A1(void)
{
   /*
    * Checking TA0IV clears the interrupt flag
    */
    switch (__even_in_range(TA0IV, 10))  // Efficient switch-implementation
    {
      case 0:
          break;    // No interrupt
      case 2:
          P3OUT &= ~(AIN2|BIN2);
          break;    // CCR1 capture interrupt
      case 4:
          P3OUT &= ~(AIN1|BIN1);
          break;    // CCR2 capture interrupt
      case 6:
          break;    // Reserved
      case 8:
          break;    // Reserved
      case 10:
          if (gBrushedObj.ain1 > MIN_PWM_DUTY && gBrushedObj.ain1 < MAX_PWM_DUTY)
          {
              P3OUT |= (AIN1|BIN1);
          }
          if (gBrushedObj.ain2 > MIN_PWM_DUTY && gBrushedObj.ain2 < MAX_PWM_DUTY)
          {
              P3OUT |= (AIN2|BIN2);
          }

          break;    // Timer Overflow
    } // end Timer0_A1()

}

/*
 * USCI Receive interrupt to receive the MDBU data packet
 */
#pragma vector=USCIAB0RX_VECTOR
__interrupt void USCI0RX_ISR(void)
{
    cmdRxIsr();
    P3OUT ^= GUI_STATUS;                       // Toggle GUI_STATUS LED for every UART RX interrupt
} // end USCI0RX_ISR()

/*
 * USCI transmit interrupt to transmit the MDBU data packet
 */
#pragma vector=USCIAB0TX_VECTOR
__interrupt void USCI0TX_ISR(void)
{
  P3OUT ^= GUI_STATUS;                          // for code debugging *UART TX interrupt not enabled in this firmware
} // end USCI0TX_ISR()

/*
 * Status LED blink rate derived using the watchdog timer
 */
#pragma vector=WDT_VECTOR
__interrupt void WatchDog_Timer(void)
{
    ++OFCount;
    if (!(OFCount % 200))
        P1OUT ^= STATUS;                       // Blinking status LED indicates DRV3946Q1EVM firmware is executing correctly
    __bic_SR_register_on_exit(LPM0_bits);      // Clear LPM0 bits, wake up from LPM0 upon ISR exit
} // end WatchDog_Timer()

#pragma vector = PORT1_VECTOR
__interrupt void PORT1_ISR(void)
{
    if (P1IFG & PUSH)                          // Push button function not implemented in this firmware version
    {
        P1IFG &= ~PUSH;                        // Clear PUSH IFG
        //P1OUT ^= STATUS;                     // for debugging code
    }
} // end PORT1_ISR()

/*
 * ADC10 interrupt Service Routine(result event) to read current value IPROPI1 & IPROPI2
 */
#pragma vector=ADC10_VECTOR
__interrupt void ADC10_ISR (void)
{
    //P1OUT &= ~TP13;                          // for TP13 probing with oscilloscope, ADC not enabled in this firmware version
} // end ADC10_ISR()

/*
 * TRAP: Trap Service routine for all unused ISRs
 */
#pragma vector=TIMER0_A0_VECTOR, TIMER1_A0_VECTOR, PORT2_VECTOR, \
        COMPARATORA_VECTOR, NMI_VECTOR
__interrupt void Trap_ISR(void)
{
    /*
     * disable drivers and oscillate between STATUS and FAULT_A and FAULT_B
     * LEDs to indicate the firmware has crashed
     * Note: nFAULT is active low, and STATUS is active high
     */
    set_nSLEEP_lo;                             // Disable driver
    P1OUT &= ~STATUS;                          // Status LED off

    while (1)
    {
        P1OUT |= STATUS;
        _delay_cycles(10000000);
    }
} // end Trap_ISR()
